(function() {
  var changeFileExtension, colors, cp, dirWatcher, echoUsageAndExit, fs, path, run, runTestFile, util;
  colors = require('colors');
  cp = require('child_process');
  dirWatcher = require('dir-watcher');
  fs = require('fs');
  path = require('path');
  util = require('util');
  changeFileExtension = function(filePath, oldExtension, newExtension) {
    if (path.extname(filePath) !== oldExtension) {
      throw 'changeFileExtension Error: Path ' + filePath + ' does not have the extension ' + oldExtension + '.';
    }
    return path.resolve(path.dirname(filePath), path.basename(filePath, oldExtension) + newExtension);
  };
  echoUsageAndExit = function() {
    console.warn('Usage: coffee-new <type> <app name> <author> <description>');
    console.warn('<type> can be "cli" or "library"');
    return process.exit(-1);
  };
  run = function(cmd, args, finishedCallback, outputToStdOut) {
    var p;
    if (outputToStdOut == null) {
      outputToStdOut = true;
    }
    p = cp.spawn(cmd, args);
    if (outputToStdOut) {
      p.stdout.on('data', function(data) {
        return process.stdout.write(data);
      });
      p.stderr.on('data', function(data) {
        return process.stderr.write(data);
      });
    }
    return p.on('exit', function(exitCode) {
      var exitString;
      if (outputToStdOut) {
        exitString = cmd + ' process exited with code ' + exitCode;
        if (exitCode !== 0) {
          util.log(exitString.red);
        } else {
          util.log(exitString.green);
        }
      }
      if (finishedCallback != null) {
        return finishedCallback(exitCode);
      }
    });
  };
  runTestFile = function(file, finishedCallback, outputToStdOut) {
    if (outputToStdOut == null) {
      outputToStdOut = true;
    }
    if (outputToStdOut) {
      util.log('Running test file '.yellow + file.yellow.bold);
    }
    return run('expresso', [file], finishedCallback, outputToStdOut);
  };
  exports.autoTest = function(persist, outputToStdOut, runTestFileCallback) {
    var cwd, srcWatcher, testWatcher;
    if (persist == null) {
      persist = true;
    }
    if (outputToStdOut == null) {
      outputToStdOut = true;
    }
    if (!(runTestFileCallback != null)) {
      runTestFileCallback = function(file) {
        return runTestFile(file, void 0, outputToStdOut);
      };
    }
    if (outputToStdOut) {
      util.log('Watching...');
    }
    cwd = process.cwd();
    dirWatcher.setup(persist);
    srcWatcher = dirWatcher.create(function(entry) {
      var testFile;
      if (path.extname(entry) === '.coffee') {
        testFile = dirWatcher.getMirrorPath(cwd, entry, 'src', 'test');
        testFile = changeFileExtension(testFile, '.coffee', '.test.coffee');
        return path.exists(testFile, function(exists) {
          var relativeTestPath;
          relativeTestPath = dirWatcher.getRelativePath(cwd, testFile);
          if (exists) {
            return runTestFileCallback(relativeTestPath);
          } else {
            console.log(relativeTestPath);
            if (outputToStdOut) {
              return util.log(('Test file ' + relativeTestPath + " doesn't exist.").red);
            }
          }
        });
      }
    });
    srcWatcher.watchDirectoryTreeSync('src');
    testWatcher = dirWatcher.create(function(entry) {
      var relativeTestPath;
      if (path.extname(entry) === '.coffee') {
        relativeTestPath = dirWatcher.getRelativePath(cwd, entry);
        return runTestFileCallback(relativeTestPath);
      }
    });
    return testWatcher.watchDirectoryTreeSync('test');
  };
  exports.build = function(finishedCallback, outputToStdOut) {
    if (outputToStdOut == null) {
      outputToStdOut = true;
    }
    if (outputToStdOut) {
      util.log('Building...');
    }
    return run('coffee', ['-c', '-o', 'lib/', 'src/'], function(exitCode) {
      if (outputToStdOut) {
        util.log('Building complete.');
      }
      if (finishedCallback != null) {
        return finishedCallback(exitCode);
      }
    }, outputToStdOut);
  };
  exports.changeFileExtension = changeFileExtension;
  exports.cleanLibDirectory = function(finishedCallback) {
    dirWatcher.rmRecursiveSync('lib');
    if (finishedCallback != null) {
      return finishedCallback();
    }
  };
  exports.run = run;
  exports.runTestFile = runTestFile;
  exports.runTestSuite = function(finishedCallback, executeTests) {
    var cwd, entry, testFileList;
    if (executeTests == null) {
      executeTests = true;
    }
    cwd = process.cwd();
    testFileList = (function() {
      var _i, _len, _ref, _results;
      _ref = dirWatcher.walkDirectoryTreeSync('test');
      _results = [];
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        entry = _ref[_i];
        if (dirWatcher.isFileSync(entry)) {
          _results.push(dirWatcher.getRelativePath(cwd, entry));
        }
      }
      return _results;
    })();
    if (executeTests) {
      run('expresso', testFileList, finishedCallback);
    }
    return testFileList;
  };
  exports.validateRootAccess = function(warnAndExit) {
    var isRoot;
    if (warnAndExit == null) {
      warnAndExit = true;
    }
    isRoot = process.env['USER'] === 'root';
    if (!isRoot && warnAndExit) {
      console.log('Must be run as root');
      return process.exit(-1);
    } else {
      return isRoot;
    }
  };
  exports.start = function() {
    var additionalPackageCode, appAuthor, appCode, appDescription, appName, appType;
    if (process.argv.length !== 6) {
      console.warn('Error: Incorrect number of parameters');
      echoUsageAndExit();
    }
    appType = process.argv[2];
    appName = process.argv[3];
    appAuthor = process.argv[4];
    appDescription = process.argv[5];
    if (appType !== 'cli' && appType !== 'library') {
      console.warn('Error: First parameter <type> is incorrect');
      echoUsageAndExit();
    }
    if (path.existsSync("" + appName + "/")) {
      console.warn("Error: Directory " + appName + " already exists");
      process.exit(-1);
    }
    if (appType === 'cli') {
      appCode = "# requires\ncolors = require 'colors'\nprogram = require 'commander'\n\n# globals\n\n# private functions\n\n# public functions\nexports.start = () ->\n	program\n		.version('0.0.1')\n		.option('-o, --option <option>', 'option description here')\n\n	program\n		.command('doCommand <arg1> <arg2>')\n		.description('command description here')\n		.action (arg1, arg2) ->\n			# code goes here\n	\n	program.parse process.argv\n	# if required, you can also parse program.args";
      additionalPackageCode = ",\n  \"dependencies\": {\n    \"colors\": \">=0.5.0\",\n    \"commander\": \">=0.1.0\"\n  },\n  \"bin\": {\n    \"" + appName + "\": \"./bin/" + appName + "\"\n  }";
    } else if (appType === 'library') {
      appCode = "# requires\n\n# globals\n\n# private functions\n\n# public functions\nexports.doSomething = ->\n	# code goes here";
      additionalPackageCode = ",\n  \"dependencies\": {\n  }";
    } else {
      throw 'appType ' + appType + " doesn't exist.";
    }
    fs.mkdirSync("" + appName + "/", 0744);
    fs.writeFile("" + appName + "/.gitignore", "npm-debug.log\nlib/*\nnode_modules/*");
    fs.writeFile("" + appName + "/.npmignore", ".git*\nCakefile\nnpm-debug.log\nsrc/\ntest/\nREADME.md");
    if (appType === 'cli') {
      fs.mkdir("" + appName + "/bin/", 0744, function() {
        return fs.writeFile("" + appName + "/bin/" + appName, "#!/usr/bin/env node\nrequire('../lib/" + appName + "').start();", function(err) {
          return fs.chmod("" + appName + "/bin/" + appName, 0755);
        });
      });
    }
    fs.writeFile("" + appName + "/Cakefile", "cn = require 'coffee-new'\n\ntask 'autotest', 'watch & test when changes are detected', (options) ->\n	cn.autoTest()\n\ntask 'build', 'compile all files in src/ to be placed in lib/', (options) ->\n	cn.build()\n\ntask 'clean', 'delete everything in lib/', (options) ->\n	cn.cleanLibDirectory()\n\ntask 'install', 'build & run npm -g install', (options) ->\n	cn.validateRootAccess()\n	cn.build -> cn.run 'npm', ['-g', 'install']\n\ntask 'publish', 'test, build & publish to npm', (options) ->\n	cn.validateRootAccess()\n	cn.runTestSuite (code) ->\n		if code == 0\n			cn.build -> cn.run 'npm', ['publish']\n\ntask 'test', 'run the entire test suite', (options) ->\n	cn.runTestSuite()\n\ntask 'uninstall', 'run npm -g uninstall', (options) ->\n	cn.validateRootAccess()\n	cn.run 'npm', ['-g', 'uninstall', '" + appName + "']");
    fs.mkdir("" + appName + "/lib/", 0744);
    fs.mkdir("" + appName + "/src/", 0744, function() {
      return fs.writeFile("" + appName + "/src/" + appName + ".coffee", appCode);
    });
    fs.mkdir("" + appName + "/test/", 0744, function() {
      return fs.writeFile("" + appName + "/test/" + appName + ".test.coffee", "app = require '../src/" + appName + "'\nshould = require 'should'\n\nexports.testTodo = ->\n	1.should.be.equal 1");
    });
    fs.writeFile("" + appName + "/README.md", "# " + appName + "\n" + appDescription + "\n\nEnjoy!\n\n" + appAuthor);
    return fs.writeFile("" + appName + "/package.json", "{\n  \"name\": \"" + appName + "\",\n  \"description\": \"" + appDescription + "\",\n  \"author\": \"" + appAuthor + "\",\n  \"version\": \"0.0.1\",\n  \"main\": \"lib/" + appName + "\",\n  \"licenses\": [{\n    \"type\": \"MIT\",\n    \"url\": \"http://opensource.org/licenses/mit-license.php\"\n  }],\n  \"engines\": {\n    \"node\": \">=0.4.5\"\n  }" + additionalPackageCode + "\n}");
  };
}).call(this);
